"""Workspace data fixtures for tmuxp tests."""

from __future__ import annotations

from . import (
    expand1,
    expand2,
    expand_blank,
    sample_workspace,
    shell_command_before,
    shell_command_before_session,
    trickle,
)
